@@ -7,7 +7,7 @@ package Mysql::Statement;
 use strict;
 use vars qw($OPTIMIZE $VERSION $AUTOLOAD);
 
-$VERSION = '1.21_06';
+$VERSION = '1.21_07';
 
 $OPTIMIZE = 0; # controls, which optimization we default to
 
@@ -46,26 +46,53 @@ sub dataseek ($$) {
 
 sub numrows { my($self) = shift; $self->rows() }
 sub numfields { my($self) = shift; $self->{'NUM_OF_FIELDS'} }
-sub affectedrows { my($self) = shift; $self->{'affected_rows'} }
-sub insertid { my($self) = shift; $self->{'insertid'} }
 sub arrAttr ($$) {
     my($self, $attr) = @_;
-    my($arr) = $self->{$attr};
+    my $arr = $self->{$attr};
     wantarray ? @$arr : $arr
 }
-sub table ($) { shift->arrAttr('table') }
+sub table ($) { shift->arrAttr('mysql_table') }
 sub name ($) { shift->arrAttr('NAME') }
+*affectedrows = \&numrows;
+sub insertid { my($self) = shift; $self->{'mysql_insertid'} }
 sub type ($) { shift->arrAttr('mysql_type') }
-sub isnotnull ($) { shift->arrAttr('is_not_null') }
-sub isprikey ($) { shift->arrAttr('is_pri_key') }
-sub isnum ($) { shift->arrAttr('is_num') }
-sub isblob ($) { shift->arrAttr('is_blob') }
-sub length ($) { shift->arrAttr('length') }
+sub isnotnull ($) {
+    my $arr = [map {!$_} @{shift()->{'NULLABLE'}}];
+    wantarray ? @$arr : $arr;
+}
+sub isprikey ($) { shift->arrAttr('mysql_is_pri_key') }
+sub isnum ($) { shift->arrAttr('mysql_is_num') }
+sub isblob ($) { shift->arrAttr('mysql_is_blob') }
+sub length ($) { shift->arrAttr('PRECISION') }
 
 sub maxlength  {
     my $sth = shift;
     my $result;
-    $result = $sth->fetchinternal('maxlength');
+    if (!($result = $sth->{'mysql_maxlength'})) {
+	$result = [];
+	my ($l);
+	for (0..$sth->numfields-1) {
+	    $$result[$_] = 0;
+	}
+	$sth->dataseek(0);
+	my($col, @row, $i);
+	while (@row = $sth->fetchrow) {
+	    for ($i = 0;  $i < @row;  $i++) {
+		$col = $row[$i];
+		my($s) = defined $col ? unctrl($col) : "NULL";
+		# New in 2.0: a string is longer than it should be
+		if (defined &Msql::TEXT_TYPE  &&
+		    $sth->type->[$i] == &Msql::TEXT_TYPE &&
+		    CORE::length($s) > $sth->length->[$i] + 5) {
+		    my $l = CORE::length($col);
+		    substr($s,$sth->length->[$i]) = "...($l)";
+		}
+		if (CORE::length($s) > $$result[$i]) {
+		    $$result[$i] = CORE::length($s);
+		}
+	    }
+	}
+    }
     return wantarray ? @$result : $result;
 }
 
@@ -100,10 +127,9 @@ sub unctrl {
 sub optimize {
     my($self,$arg) = @_;
     if (defined $arg) {
-	return $self->{'OPTIMIZE'} = $arg;
-    } else {
-	return $self->{'OPTIMIZE'} ||= $OPTIMIZE;
+	$OPTIMIZE = $arg;
     }
+    $OPTIMIZE;
 }
 
 sub as_string {
